//-----------------------------------------------------------------------------
// File: objects.cpp
//
// Desc: Example code showing how to rotating two objects independantly 
//       in a scene and translating and rotating a camera
//
//       Note: This code uses the D3D Framework helper library.
//
// Copyright (c) 1997-1999 Microsoft Corporation. All rights reserved.
// Partly copyright (c) 1999 - 2001 Wolfgang Engel wolf@direct3d.net
//-----------------------------------------------------------------------------
#define STRICT
#define D3D_OVERLOADS
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "D3DTextr.h"
#include "D3DUtil.h"
#include "D3DMath.h"
#include "D3DApp.h"
#include "dinput.h"


//-----------------------------------------------------------------------------
// Name: struct Object
// Desc: Structure for holding data for each object
//-----------------------------------------------------------------------------
struct Object
{
    D3DVECTOR   vLoc;
    FLOAT       fYaw, fPitch, fRoll; 
    FLOAT       r, g, b;    // Color of the object
    D3DMATRIX   matLocal;
};



//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class provides just about all the
//       functionality we want, so we're just supplying stubs to interface with
//       the non-C++ functions of the app.
//-----------------------------------------------------------------------------
class CMyD3DApplication : public CD3DApplication
{
    D3DVERTEX  m_pvObjectVertices[16];
    WORD       m_pwObjectIndices[30];
    Object     m_pObjects[2];

    
	FLOAT m_fStartTimeKey,     // Time reference for calculations
		  m_fTimeElapsed;

    static HRESULT ConfirmDevice( DDCAPS* pddDriverCaps, 
                                  D3DDEVICEDESC7* pd3dDeviceDesc );
protected:
    HRESULT OneTimeSceneInit();
    HRESULT InitDeviceObjects();
    HRESULT DeleteDeviceObjects();
    HRESULT Render();
    HRESULT FrameMove( FLOAT fTimeKey );
    HRESULT FinalCleanup();

public:
	LPDIRECTINPUT7       g_Keyboard_pDI;            // The DInput object
	LPDIRECTINPUTDEVICE2 g_Keyboard_pdidDevice2;    // The DIDevice2 interface
    
	CMyD3DApplication();

	// dinput methods
	HRESULT CreateDInput( HWND hWnd );
	VOID DestroyInputDevice();
	VOID UpdateControls();
	HRESULT CreateInputDevice( HWND hWnd, 
							   LPDIRECTINPUT7       pDI,
							   LPDIRECTINPUTDEVICE2 pDIdDevice, 
							   GUID guidDevice,
							   const DIDATAFORMAT* pdidDataFormat, 
							   DWORD dwFlags );
	VOID DestroyDInput();

	// utitlity methods
    HWND Get_hWnd()	{ return m_hWnd; };

};



//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Idle time is used to render the scene.
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR strCmdLine, INT )
{
    CMyD3DApplication d3dApp;

    if( FAILED( d3dApp.Create( hInst, strCmdLine ) ) )
        return 0;

	// Create the DInput object
	if( FAILED(d3dApp.CreateDInput( d3dApp.Get_hWnd() ) ) )
        return 0;
   
    // Create a keyboard device
    if( FAILED(d3dApp.CreateInputDevice( d3dApp.Get_hWnd(), 
										 d3dApp.g_Keyboard_pDI, 
										 d3dApp.g_Keyboard_pdidDevice2, 
										 GUID_SysKeyboard, 
										 &c_dfDIKeyboard,
										 DISCL_NONEXCLUSIVE | DISCL_FOREGROUND)))
	    return 0;

	return d3dApp.Run();
}

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
CMyD3DApplication::CMyD3DApplication()
                  :CD3DApplication()
{
    m_strWindowTitle  = TEXT("First Steps To Animation");
    m_bAppUseZBuffer  = TRUE;
    m_bAppUseStereo   = FALSE;
    m_bShowStats      = TRUE;
    m_fnConfirmDevice = ConfirmDevice;

}


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::OneTimeSceneInit()
{
    // Points and normals which make up a object geometry
    D3DVECTOR p1 = D3DVECTOR( 0.00f, 0.00f, 0.50f );
    D3DVECTOR p2 = D3DVECTOR( 0.50f, 0.00f,-0.50f );
    D3DVECTOR p3 = D3DVECTOR( 0.15f, 0.15f,-0.35f );
    D3DVECTOR p4 = D3DVECTOR(-0.15f, 0.15f,-0.35f );
    D3DVECTOR p5 = D3DVECTOR( 0.15f,-0.15f,-0.35f );
    D3DVECTOR p6 = D3DVECTOR(-0.15f,-0.15f,-0.35f );
    D3DVECTOR p7 = D3DVECTOR(-0.50f, 0.00f,-0.50f );
    D3DVECTOR n1 = Normalize( D3DVECTOR( 0.2f, 1.0f, 0.0f ) );
    D3DVECTOR n2 = Normalize( D3DVECTOR( 0.1f, 1.0f, 0.0f ) );
    D3DVECTOR n3 = Normalize( D3DVECTOR( 0.0f, 1.0f, 0.0f ) );
    D3DVECTOR n4 = Normalize( D3DVECTOR(-0.1f, 1.0f, 0.0f ) );
    D3DVECTOR n5 = Normalize( D3DVECTOR(-0.2f, 1.0f, 0.0f ) );
    D3DVECTOR n6 = Normalize( D3DVECTOR(-0.4f, 0.0f, -1.0f ) );
    D3DVECTOR n7 = Normalize( D3DVECTOR(-0.2f, 0.0f, -1.0f ) );
    D3DVECTOR n8 = Normalize( D3DVECTOR( 0.2f, 0.0f, -1.0f ) );
    D3DVECTOR n9 = Normalize( D3DVECTOR( 0.4f, 0.0f, -1.0f ) );

    // Vertices for the top
    m_pvObjectVertices[ 0] = D3DVERTEX( p1, n1, 0.000f, 0.500f );
    m_pvObjectVertices[ 1] = D3DVERTEX( p2, n2, 0.500f, 1.000f );
    m_pvObjectVertices[ 2] = D3DVERTEX( p3, n3, 0.425f, 0.575f );
    m_pvObjectVertices[ 3] = D3DVERTEX( p4, n4, 0.425f, 0.425f );
    m_pvObjectVertices[ 4] = D3DVERTEX( p7, n5, 0.500f, 0.000f );

    // Vertices for the bottom
    m_pvObjectVertices[ 5] = D3DVERTEX( p1, -n5, 1.000f, 0.500f );
    m_pvObjectVertices[ 6] = D3DVERTEX( p2, -n4, 0.500f, 1.000f );
    m_pvObjectVertices[ 7] = D3DVERTEX( p5, -n3, 0.575f, 0.575f );
    m_pvObjectVertices[ 8] = D3DVERTEX( p6, -n2, 0.575f, 0.425f );
    m_pvObjectVertices[ 9] = D3DVERTEX( p7, -n1, 0.500f, 0.000f );

    // Vertices for the  rear
    m_pvObjectVertices[10] = D3DVERTEX( p2, n6, 0.500f, 1.000f );
    m_pvObjectVertices[11] = D3DVERTEX( p3, n7, 0.425f, 0.575f );
    m_pvObjectVertices[12] = D3DVERTEX( p4, n8, 0.425f, 0.425f );
    m_pvObjectVertices[13] = D3DVERTEX( p7, n9, 0.500f, 0.000f );
    m_pvObjectVertices[14] = D3DVERTEX( p6, n8, 0.575f, 0.425f );
    m_pvObjectVertices[15] = D3DVERTEX( p5, n7, 0.575f, 0.575f );

    // Vertex indices for the object
    m_pwObjectIndices[ 0] =  0; m_pwObjectIndices[ 1] =  1; m_pwObjectIndices[ 2] =  2;
    m_pwObjectIndices[ 3] =  0; m_pwObjectIndices[ 4] =  2; m_pwObjectIndices[ 5] =  3;
    m_pwObjectIndices[ 6] =  0; m_pwObjectIndices[ 7] =  3; m_pwObjectIndices[ 8] =  4;
    m_pwObjectIndices[ 9] =  5; m_pwObjectIndices[10] =  7; m_pwObjectIndices[11] =  6;
    m_pwObjectIndices[12] =  5; m_pwObjectIndices[13] =  8; m_pwObjectIndices[14] =  7;
    m_pwObjectIndices[15] =  5; m_pwObjectIndices[16] =  9; m_pwObjectIndices[17] =  8;
    m_pwObjectIndices[18] = 10; m_pwObjectIndices[19] = 15; m_pwObjectIndices[20] = 11;
    m_pwObjectIndices[21] = 11; m_pwObjectIndices[22] = 15; m_pwObjectIndices[23] = 12;
    m_pwObjectIndices[24] = 12; m_pwObjectIndices[25] = 15; m_pwObjectIndices[26] = 14;
    m_pwObjectIndices[27] = 12; m_pwObjectIndices[28] = 14; m_pwObjectIndices[29] = 13;

	// yellow object
	m_pObjects[0].vLoc   = D3DVECTOR(-1.0f, 0.0f, 0.0f);
    m_pObjects[0].fYaw   = 0.0f;
    m_pObjects[0].fPitch = 0.0f;
    m_pObjects[0].fRoll  = 0.0f;
    m_pObjects[0].r      = 1.0f;
    m_pObjects[0].g      = 0.92f;
    m_pObjects[0].b      = 0.0f;

	// red object
	m_pObjects[1].vLoc   = D3DVECTOR(1.0f, 0.0f, 0.0f);
    m_pObjects[1].fYaw   = 0.0f;
    m_pObjects[1].fPitch = 0.0f;
    m_pObjects[1].fRoll  = 0.0f;
    m_pObjects[1].r      = 1.0f;
    m_pObjects[1].g      = 0.0f;
    m_pObjects[1].b      = 0.27f;

    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::Render()
{
    D3DMATERIAL7 mtrl;

    // Clear the viewport
    m_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
                         0x00000000, 1.0f, 0L );

    // Begin the scene
    if( FAILED( m_pd3dDevice->BeginScene() ) )
        return S_OK; // Don't return a "fatal" error

		// yellow object
		// Set the color for the object
		D3DUtil_InitMaterial( mtrl, m_pObjects[0].r, m_pObjects[0].g, m_pObjects[0].b );
		m_pd3dDevice->SetMaterial( &mtrl );

        // Apply the object's local matrix
        m_pd3dDevice->SetTransform(D3DTRANSFORMSTATE_WORLD, &m_pObjects[0].matLocal );

        // Draw the object
        m_pd3dDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, D3DFVF_VERTEX,
                                            m_pvObjectVertices, 16,
                                            m_pwObjectIndices, 30, 0 );

		// red object
        // Set the color for the object
        D3DUtil_InitMaterial( mtrl, m_pObjects[1].r, m_pObjects[1].g, m_pObjects[1].b );
        m_pd3dDevice->SetMaterial( &mtrl );

        // Apply the object's local matrix
        m_pd3dDevice->SetTransform(D3DTRANSFORMSTATE_WORLD, &m_pObjects[1].matLocal );

        // Draw the object
        m_pd3dDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, D3DFVF_VERTEX,
                                            m_pvObjectVertices, 16,
                                            m_pwObjectIndices, 30, 0 );


    // End the scene.
    m_pd3dDevice->EndScene();

    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InitDeviceObjects()
{
    D3DMATERIAL7 mtrl;
    D3DUtil_InitMaterial( mtrl, 1.0f, 1.0f, 1.0f );
    m_pd3dDevice->SetMaterial( &mtrl );

    m_pd3dDevice->SetRenderState( D3DRENDERSTATE_ZENABLE, TRUE); //D3DZB_USEW );
	
/*	// wireframe object
    m_pd3dDevice->SetRenderState( D3DRENDERSTATE_FILLMODE, D3DFILL_WIREFRAME );

    // no backface culling
	m_pd3dDevice->SetRenderState(D3DRENDERSTATE_CULLMODE, 
                            D3DCULL_NONE);
*/

    // Set the transform matrices
    D3DVIEWPORT7 vp;
    m_pd3dDevice->GetViewport(&vp);
    FLOAT fAspect = ((FLOAT)vp.dwHeight) / vp.dwWidth;

    D3DMATRIX matProj;
    D3DUtil_SetProjectionMatrix( matProj, 0.75f, fAspect, 1.0f, 1000.0f );
    m_pd3dDevice->SetTransform( D3DTRANSFORMSTATE_PROJECTION, &matProj );


    // Set up the lights
    m_pd3dDevice->SetRenderState( D3DRENDERSTATE_AMBIENT, 0x0c0c0c0c);

    if( m_pDeviceInfo->ddDeviceDesc.dwVertexProcessingCaps &
                       D3DVTXPCAPS_DIRECTIONALLIGHTS )
    {
        D3DLIGHT7 light;

        if( m_pDeviceInfo->ddDeviceDesc.dwMaxActiveLights > 0 )
        {
            D3DUtil_InitLight( light, D3DLIGHT_DIRECTIONAL, 0.5f, -1.0f, 0.3f );
            m_pd3dDevice->SetLight( 0, &light );
            m_pd3dDevice->LightEnable( 0, TRUE );
        }

        if( m_pDeviceInfo->ddDeviceDesc.dwMaxActiveLights > 1 )
        {
            D3DUtil_InitLight( light, D3DLIGHT_DIRECTIONAL, 0.5f, 1.0f, 1.0f );
            light.dcvDiffuse.r = 0.5f;
            light.dcvDiffuse.g = 0.5f;
            light.dcvDiffuse.b = 0.5f;
            m_pd3dDevice->SetLight( 1, &light );
            m_pd3dDevice->LightEnable( 1, TRUE );
        }

        m_pd3dDevice->SetRenderState( D3DRENDERSTATE_LIGHTING, TRUE );
    }

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FinalCleanup()
{
	// release keyboard stuff
	DestroyInputDevice();

    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exitting, or the device is being changed,
//       this function deletes any device dependant objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::DeleteDeviceObjects()
{

   return S_OK;
}

//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device intialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::ConfirmDevice( DDCAPS* pddDriverCaps,
                                          D3DDEVICEDESC7* pd3dDeviceDesc )
{
    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: CreateDInput()
// Desc: Initialize the DirectInput objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::CreateDInput( HWND hWnd )
{
	// keyboard
	if( FAILED(DirectInputCreateEx( (HINSTANCE)GetWindowLong( hWnd, GWL_HINSTANCE ),
                                   DIRECTINPUT_VERSION, 
								   IID_IDirectInput7, 
								   (LPVOID*) &g_Keyboard_pDI, NULL) ) )
        return 0;
	
    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: CreateInputDevice()
// Desc: Create a DirectInput device.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::CreateInputDevice( HWND hWnd, 
								   LPDIRECTINPUT7       pDI,	
								   LPDIRECTINPUTDEVICE2 pDIdDevice, 
								   GUID guidDevice,
								   const DIDATAFORMAT* pdidDataFormat, DWORD dwFlags )
{

    // Obtain an interface to the input device
    if( FAILED( pDI->CreateDeviceEx( guidDevice, IID_IDirectInputDevice2,
		                               (VOID**)&pDIdDevice, NULL ) ) )
        return 0;

    // Set the device data format
    if( FAILED( pDIdDevice->SetDataFormat( pdidDataFormat ) ) )
        return 0;

    // Set the cooperativity level
    if( FAILED( pDIdDevice->SetCooperativeLevel( hWnd, dwFlags ) ) )
        return 0;

	if(guidDevice == GUID_SysKeyboard)
		g_Keyboard_pdidDevice2 = pDIdDevice;

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FrameMove (FLOAT fTimeKey)
{
    
	BYTE diks[256]; // DInput keyboard state buffer
    ZeroMemory( diks, sizeof(diks) );

	g_Keyboard_pdidDevice2->Acquire();
	g_Keyboard_pdidDevice2->GetDeviceState( sizeof(diks), &diks );

	// timing code: 
	// the object should move/rotate in the same speed 
	// at every possible fps
	const cTimeScale = 5;
	
	// calculate elapsed time
	m_fTimeElapsed=(fTimeKey-m_fStartTimeKey) * cTimeScale;
	
	// store last time
	m_fStartTimeKey=fTimeKey;


	// yellow object
	if (diks[DIK_J] &&0x80) // j key
		m_pObjects[0].fRoll -= 1.0f * m_fTimeElapsed;

    if (diks[DIK_L] &&0x80)  // l key
		m_pObjects[0].fRoll += 1.0f * m_fTimeElapsed;

	if (diks[DIK_I] &&0x80) // i key
		m_pObjects[0].fPitch -= 1.0f * m_fTimeElapsed;

    if (diks[DIK_K] &&0x80) // k key       
		m_pObjects[0].fPitch += 1.0f * m_fTimeElapsed;


    D3DMATRIX matWorld;

    // object matrix for yellow object
	D3DUtil_SetTranslateMatrix( matWorld, m_pObjects[0].vLoc );

    D3DMATRIX matTemp, matRotateX, matRotateY, matRotateZ;
    D3DUtil_SetRotateYMatrix( matRotateY, -m_pObjects[0].fYaw );
    D3DUtil_SetRotateXMatrix( matRotateX, -m_pObjects[0].fPitch );
    D3DUtil_SetRotateZMatrix( matRotateZ, -m_pObjects[0].fRoll );
    D3DMath_MatrixMultiply( matTemp, matRotateX, matRotateY );
    D3DMath_MatrixMultiply( matTemp, matRotateZ, matTemp );
    D3DMath_MatrixMultiply( matWorld, matTemp, matWorld );

    m_pObjects[0].matLocal = matWorld;

    
	// red object
    if (diks[DIK_D] &&0x80)   // Key d
		m_pObjects[1].fRoll -= 1.0f * m_fTimeElapsed;

    if (diks[DIK_A] &&0x80)   // Key a
		m_pObjects[1].fRoll += 1.0f * m_fTimeElapsed;
	
	if (diks[DIK_S] &&0x80)   // Key s
		m_pObjects[1].fPitch -= 1.0f * m_fTimeElapsed;

    if (diks[DIK_W] &&0x80)   // Key w     
		m_pObjects[1].fPitch += 1.0f * m_fTimeElapsed;

	// object matrix for red object
	D3DUtil_SetTranslateMatrix( matWorld, m_pObjects[1].vLoc );

    D3DUtil_SetRotateYMatrix( matRotateY, -m_pObjects[1].fYaw );
    D3DUtil_SetRotateXMatrix( matRotateX, -m_pObjects[1].fPitch );
    D3DUtil_SetRotateZMatrix( matRotateZ, -m_pObjects[1].fRoll );
    D3DMath_MatrixMultiply( matTemp, matRotateX, matRotateY );
    D3DMath_MatrixMultiply( matTemp, matRotateZ, matTemp );
    D3DMath_MatrixMultiply( matWorld, matTemp, matWorld );

    m_pObjects[1].matLocal = matWorld;

	//************************************************************
	// camera stuff
	//************************************************************
	static D3DVECTOR vLook=D3DVECTOR(0.0f,0.0f,1.0);
	static D3DVECTOR vUp=D3DVECTOR(0.0f,1.0f,0.0f);
	static D3DVECTOR vRight=D3DVECTOR(1.0f,0.0f,0.0f);
	static D3DVECTOR vPos=D3DVECTOR(0.0f,0.0f,-5.0f);
	FLOAT fPitch,fYaw,fRoll;
	fPitch = fYaw = fRoll = 0.0f;

	FLOAT fspeed= 1.0f * m_fTimeElapsed;

	// fPitch
    if (diks[DIK_UP] &&0x80) 
		fPitch=-0.3f * m_fTimeElapsed;

    if (diks[DIK_DOWN] &&0x80)
		fPitch=+0.3f * m_fTimeElapsed;

	// fYaw
    if (diks[DIK_C] &&0x80) // c key
		fYaw=-0.3f * m_fTimeElapsed;

    if (diks[DIK_X] &&0x80) // x key
		fYaw=+0.3f * m_fTimeElapsed;

	// fRoll
    if (diks[DIK_LEFT]	&& 0x80)
		fRoll=-0.3f * m_fTimeElapsed;

    if (diks[DIK_RIGHT]	&& 0x80) 
		fRoll=+0.3f * m_fTimeElapsed;

	 // camera forward
    if (diks[DIK_HOME] &&0x80 )   // Key HOME     
	{
		vPos.x+=fspeed*vLook.x;
		vPos.y+=fspeed*vLook.y;
		vPos.z+=fspeed*vLook.z;
	}

	// camera back
    if (diks[DIK_END] &&0x80 )   // Key END     
	{
		vPos.x-=fspeed*vLook.x;
		vPos.y-=fspeed*vLook.y;
		vPos.z-=fspeed*vLook.z;
	}

	vLook = Normalize(vLook);
    vRight = CrossProduct( vUp, vLook); // Cross Produkt of the UP and LOOK Vector
	vRight = Normalize (vRight);
	vUp = CrossProduct (vLook, vRight); // Cross Produkt of the RIGHT and LOOK Vector
	vUp = Normalize(vUp);

    // Matrices for pitch, yaw and roll
	// This creates a rotation matrix around the viewers RIGHT vector. 
	D3DMATRIX matPitch, matYaw, matRoll;
    D3DUtil_SetRotationMatrix(matPitch, vRight, fPitch); 
	// Creates a rotation matrix around the viewers UP vector. 
    D3DUtil_SetRotationMatrix(matYaw, vUp, fYaw );
	// Creates a rotation matrix around the viewers LOOK vector. 
    D3DUtil_SetRotationMatrix(matRoll, vLook, fRoll);

	// now multiply these vectors with the matrices we've just created. 
	// First we rotate the LOOK & RIGHT Vectors about the UP Vector
    D3DMath_VectorMatrixMultiply(vLook , vLook, matYaw);
    D3DMath_VectorMatrixMultiply(vRight, vRight,matYaw);

	// And then we rotate the LOOK & UP Vectors about the RIGHT Vector
    D3DMath_VectorMatrixMultiply(vLook , vLook, matPitch);
    D3DMath_VectorMatrixMultiply(vUp, vUp, matPitch);

	// now rotate the RIGHT & UP Vectors about the LOOK Vector
    D3DMath_VectorMatrixMultiply(vRight, vRight, matRoll);
    D3DMath_VectorMatrixMultiply(vUp, vUp, matRoll);

	D3DMATRIX view=matWorld;
	D3DUtil_SetIdentityMatrix( view );// defined in d3dutil.h and d3dutil.cpp 
	view._11 = vRight.x; view._12 = vUp.x; view._13 = vLook.x;
	view._21 = vRight.y; view._22 = vUp.y; view._23 = vLook.y;
	view._31 = vRight.z; view._32 = vUp.z; view._33 = vLook.z;
	view._41 = - DotProduct( vPos, vRight ); // dot product defined in d3dtypes.h
	view._42 = - DotProduct( vPos, vUp );
	view._43 = - DotProduct( vPos, vLook );

	m_pd3dDevice->SetTransform(D3DTRANSFORMSTATE_VIEW,&view); 
  
    return S_OK;

}

//-----------------------------------------------------------------------------
// Name: DestroyInputDevice()
// Desc: Release the DirectInput device
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::DestroyInputDevice()
{
	// keyboard
	if(g_Keyboard_pdidDevice2)
    {
        g_Keyboard_pdidDevice2->Unacquire();
        g_Keyboard_pdidDevice2->Release();
        g_Keyboard_pdidDevice2 = NULL;
    }
}

